import sys
import base64
import logging
import sys
from typing import List

from cli.ascii_table import print_file_details
from cli.client import TextToQR, Client
from cli.colors import colored

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def generate_ascii_qr_code(qr: TextToQR, margin: int = 2) -> List[str]:
    decoded = bytes(base64.b64decode(qr.c)).decode('utf-8')
    binary = ''.join(list(map(lambda b: bin(ord(b)).replace('0b', '').zfill(8), decoded)))
    binary = binary[qr.n0:]

    x = qr.x

    rows = []
    rows.extend(['  ' * (x + margin * 2) for _ in range(margin)])

    for i in range(x):
        row = binary[i * x:(i + 1) * x]
        row = ''.join(list(map(lambda c: '█' * 2 if c == '1' else '  ', row)))
        rows.append(('  ' * margin) + row + ('  ' * margin))

    rows.extend(['  ' * (x + margin * 2) for _ in range(margin)])

    return rows


def main():
    if len(sys.argv) != 2:
        print(f"Usage: qrs <filename>")
        sys.exit(-1)
    client = Client()
    response = client.upload(sys.argv[1])

    code = generate_ascii_qr_code(TextToQR(**response.jqr))
    for line in code:
        print(colored(line))

    print_file_details(response.filename, response.size, response.url)
    print()


if __name__ == "__main__":
    main()



import logging
import re
from typing import NamedTuple, Union, List
from datetime import datetime

import requests

from cli.colors import colored

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class TextToQR(NamedTuple):
    n0: int
    x: int
    c: str


class UploadResponse(NamedTuple):
    success: bool
    filename: str
    dir: str
    url: str
    errorMessage: Union[str, None]
    contentType: str
    size: int
    uploadedAt: datetime
    expiresAt: datetime
    jqr: dict


BASE_URL = "https://api2.qrshare.io"


class Client:
    def __init__(self, base_url: str = BASE_URL):
        self.base_url = base_url

    def upload(self, filename: str) -> UploadResponse:
        print(f"Uploading: {colored(filename)}...")
        with open(filename, "rb") as f:
            data = {"file": (re.split(r"/|\\", filename)[-1], f)}
            response = requests.post(f"{self.base_url}/api/upload?ddl=true", files=data)
            response_content = response.json()

            logger.debug(f"{response.status_code}\n{response_content}")

            if response.ok:
                return UploadResponse(**response_content)
            else:
                return UploadResponse(success=False, errorMessage=f"{response.status_code}: {response.reason}")

    def get_direct_download_url(self, upload_id: str) -> TextToQR:
        return f"{self.base_url}/api/v2/transfer/ddl/{upload_id}"

    def text_to_qr(self, text: str) -> TextToQR:
        response = requests.get(f"{self.base_url}/api/qr/json?text={text}").json()
        return TextToQR(**response)



from cli.colors import colored


def convert_file_size(file_size: int) -> str:
    if file_size < 1024:
        return f"{file_size:,} bytes"
    elif file_size < 1048576:
        return f"{file_size / 1024:,.2f} KB"
    elif file_size < 1073741824:
        return f"{file_size / 1048576:,.2f} MB"
    else:
        return f"{file_size / 1073741824:,.2f} GB"


def print_file_details(filename: str, size_bytes: int, url: str, prefix: str = ' ' * 4):
    names_col_len = 12
    values_col_len = max(len(url), len(filename))
    rows = ['╔' + ('═' * (names_col_len + 2)) + '╦' + ('═' * (values_col_len + 2)) + '╗']

    data = [
        ('Filename'.rjust(names_col_len), filename.ljust(values_col_len)),
        ('Size'.rjust(names_col_len), convert_file_size(size_bytes).ljust(values_col_len)),
        ('Download URL'.rjust(names_col_len), url.ljust(values_col_len))
    ]

    for (i, (name, value)) in enumerate(data):
        value_color = 'white' if i in [0, 1] else 'blue'
        rows.append('║ ' + name + ' ║ ' + colored(value, value_color, bold=True) + ' ║')
        if i != len(data) - 1:
            rows.append('╠' + ('═' * (names_col_len + 2)) + '╬' + ('═' * (values_col_len + 2)) + '╣')
    rows.append('╚' + ('═' * (names_col_len + 2)) + '╩' + ('═' * (values_col_len + 2)) + '╝')

    for row in rows:
        print(prefix + row)
